package org.tip.puck.clusternetworks;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.tip.puck.PuckException;
import org.tip.puck.alliancenets.AllianceNet;
import org.tip.puck.alliancenets.Group;
import org.tip.puck.census.chains.Chain;
import org.tip.puck.census.workers.CircuitFinder;
import org.tip.puck.graphs.ClusterPair;
import org.tip.puck.graphs.Graph;
import org.tip.puck.matrix.Matrix;
import org.tip.puck.net.Family;
import org.tip.puck.net.Gender;
import org.tip.puck.net.Individual;
import org.tip.puck.net.Net;
import org.tip.puck.partitions.Cluster;
import org.tip.puck.partitions.Partition;
import org.tip.puck.partitions.PartitionMaker;
import org.tip.puck.segmentation.Segmentation;

/**
 * 
 * @author TIP
 */
public class ClusterNetworkUtils {

	public enum LineType {
		SIMPLE_ARC,
		WEIGHTED_ARC;
	}

	

	/**
	 * 
	 * @param source
	 * @return
	 * @throws PuckException
	 */
	public static Graph<Cluster<Individual>> createAllianceNetwork(final Segmentation source, final String partitionLabel, final LineType lineType)
			throws PuckException {
		Graph<Cluster<Individual>> result;

		if ((source == null) || (partitionLabel == null)) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			Partition<Individual> partition = PartitionMaker.createRaw(source.getLabel(), source.getCurrentIndividuals(), partitionLabel);

			//
			result = new Graph<Cluster<Individual>>("Alliance Network " + source.getLabel() + " " + partitionLabel);
			for (Cluster<Individual> cluster : partition.getClusters()) {
				if (!cluster.isNull()) {
					result.addNode(cluster);
				}
			}

			//
			for (Family family : source.getCurrentFamilies()) {
				if (family.isMarried()) {
					Individual husband = family.getHusband();
					Individual wife = family.getWife();
					if ((husband != null) && (wife != null)) {
						Cluster<Individual> husbandsCluster = partition.getCluster(husband);
						Cluster<Individual> wifesCluster = partition.getCluster(wife);
						if (husbandsCluster != null && !husbandsCluster.isNull() && wifesCluster != null && !wifesCluster.isNull()) {
							if (lineType == LineType.WEIGHTED_ARC) {
								result.incArcWeight(wifesCluster, husbandsCluster);
							} else if (lineType == LineType.SIMPLE_ARC) {
								if (result.getArc(wifesCluster, husbandsCluster) == null) {
									result.addArc(wifesCluster, husbandsCluster, 1);
								}
							}
						}
					}
				}
			}
		}

		//
		return result;
	}

/*	public static Graph<Cluster<Chain>> createCircuitIntersectionNetwork1(final CircuitFinder source) throws PuckException {
		Graph<Cluster<Chain>> result;

		if (source == null) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			result = new Graph<Cluster<Chain>>("Circuit Intersection Network: " + source.getLabel());

			int z = 0;
			
			for (Cluster<Chain> coupleCluster : source.byCouplesWithoutTransformation(Gender.MALE).getClusters().toListSortedByValue()) {
				for (int i = 0; i < coupleCluster.count(); i++) {
					Cluster<Chain> typeCluster1 = source.getCircuits().getCluster(coupleCluster.getItems().get(i).standard());
					for (int j = 0; j <= i; j++) {
						Cluster<Chain> typeCluster2 = source.getCircuits().getCluster(coupleCluster.getItems().get(j).standard());
						if (!typeCluster1.isNull() && !typeCluster2.isNull()) {
							result.incEdgeWeight(typeCluster2, typeCluster1);
							if (typeCluster1.getLabel().equals("0") && (i==j)) {
								z++;
								System.out.println(z+" "+result.getEdgeWeight(result.getNode(typeCluster1), result.getNode(typeCluster1))+" "+coupleCluster.getLabel());
							}
						}
					}
				}
			}
		}
		//
		return result;
	}*/
	
	public static Graph<Cluster<Chain>> createCircuitIntersectionNetwork(final CircuitFinder source) throws PuckException {
		Graph<Cluster<Chain>> result;

		if (source == null) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			result = new Graph<Cluster<Chain>>("Circuit Intersection Network: " + source.getLabel());

			for (Cluster<Cluster<Chain>> coupleCluster : source.clustersByCouples(Gender.MALE).getClusters().toListSortedByValue()) {
				for (int i = 0; i < coupleCluster.count(); i++) {
					Cluster<Chain> typeCluster1 = coupleCluster.getItems().get(i);
					for (int j = 0; j <= i; j++) {
						Cluster<Chain> typeCluster2 = coupleCluster.getItems().get(j);
						if (!typeCluster1.isNull() && !typeCluster2.isNull()) {
							result.incEdgeWeight(typeCluster2, typeCluster1);
						}
					}
				}
			}
		}
		//
		return result;
	}

/*	public static Graph<Cluster<Chain>> createCircuitIntersectionNetwork(final Map<Chain,Set<Cluster<Chain>>> map, CircuitFinder source) throws PuckException {
		Graph<Cluster<Chain>> result;

		if (map == null) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			result = new Graph<Cluster<Chain>>("Circuit Intersection Network: " + source.getLabel());

			for (Set<Cluster<Chain>> clusters : map.values()){
				List<Cluster<Chain>> coupleCluster = new ArrayList<Cluster<Chain>>(clusters);
				for (int i = 0; i < coupleCluster.size(); i++) {
					Cluster<Chain> typeCluster1 = coupleCluster.get(i);
//					Cluster<Chain> typeCluster1 = source.getCircuits().getCluster(coupleCluster.get(i).standard());
					for (int j = 0; j < i; j++) {
						Cluster<Chain> typeCluster2 = coupleCluster.get(j);
//						Cluster<Chain> typeCluster2 = source.getCircuits().getCluster(coupleCluster.get(j).standard());
						if (!typeCluster1.isNull() && !typeCluster2.isNull()) {
							result.incEdgeWeight(typeCluster2, typeCluster1);
						}
					}
				}
			}
			
		}
		//
		return result;
	}*/
	

	/**
	 * 
	 * @param source
	 * @param birthPartitionLabel
	 * @param deathPartitionLabel
	 * @return
	 * @throws PuckException
	 */
	public static Graph<ClusterPair<Individual>> createFlowNetwork(final Segmentation source, final String sourceLabel, final String targetLabel,
			final LineType lineType) throws PuckException {
		Graph<ClusterPair<Individual>> result;

		if ((source == null) || (sourceLabel == null) || (targetLabel == null)) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			Partition<Individual> sourcePartition = PartitionMaker.createRaw(source.getLabel(), source.getCurrentIndividuals(), sourceLabel);
			Partition<Individual> targetPartition = PartitionMaker.createRaw(source.getLabel(), source.getCurrentIndividuals(), targetLabel);

			//
			result = new Graph<ClusterPair<Individual>>("Flow Network: " + source.getLabel() + " " + sourceLabel + " " + targetLabel);

			//
			for (Individual individual : source.getCurrentIndividuals()) {
				//
				Cluster<Individual> sourceCluster = sourcePartition.getCluster(individual);
				Cluster<Individual> targetCluster = targetPartition.getCluster(individual);

				//
				ClusterPair<Individual> sourcePair = new ClusterPair<Individual>(sourceCluster, targetPartition.getCluster(sourceCluster.getValue()));
				ClusterPair<Individual> targetPair = new ClusterPair<Individual>(sourcePartition.getCluster(targetCluster.getValue()), targetCluster);

				//
				if (!sourcePair.getLeft().isNull() && !targetPair.getRight().isNull()) {
					if (lineType == LineType.WEIGHTED_ARC) {
						result.incArcWeight(sourcePair, targetPair);
					} else if (lineType == LineType.SIMPLE_ARC) {
						if (result.getArc(sourcePair, targetPair) == null) {
							result.addArc(sourcePair, targetPair, 1);
						}
					}
				}
			}
		}

		//
		return result;
	}

	/**
	 * 
	 * @param source
	 * @return
	 * @throws PuckException
	 */
	public static Graph<Cluster<Individual>> createSiblingNetwork(final Segmentation source, final String partitionLabel, final LineType lineType)
			throws PuckException {
		Graph<Cluster<Individual>> result;

		if ((source == null) || (partitionLabel == null)) {
			throw new NullPointerException("Null parameter detected.");
		} else {
			//
			Partition<Individual> partition = PartitionMaker.createRaw(source.getLabel(), source.getCurrentIndividuals(), partitionLabel);

			//
			result = new Graph<Cluster<Individual>>("Sibling Network " + source.getLabel() + " " + partitionLabel);
			for (Cluster<Individual> cluster : partition.getClusters()) {
				if (!cluster.isNull()) {
					result.addNode(cluster);
				}
			}

			//
			for (Family family : source.getCurrentFamilies()) {
				for (Individual brother : family.getChildren()) {
					if (brother != null && brother.isMale() && brother.isNotSingle()) {
						for (Individual sister : family.getChildren()) {
							if (sister != null && sister.isFemale() && sister.isNotSingle()) {
								Cluster<Individual> brothersCluster = partition.getCluster(brother);
								Cluster<Individual> sistersCluster = partition.getCluster(sister);
								if (brothersCluster != null && !brothersCluster.isNull() && sistersCluster != null && !sistersCluster.isNull()) {
									if (lineType == LineType.WEIGHTED_ARC) {
										result.incArcWeight(sistersCluster, brothersCluster);
									} else if (lineType == LineType.SIMPLE_ARC) {
										if (result.getArc(sistersCluster, brothersCluster) == null) {
											result.addArc(sistersCluster, brothersCluster, 1);
										}
									}
								}
							}
						}
					}
				}
			}
		}

		//
		return result;
	}

	/**
	 * 
	 * @param source
	 * @param birthPartitionLabel
	 * @param deathPartitionLabel
	 * @return
	 * @throws PuckException
	 */
	public static Graph<Cluster<Individual>> createUnmergedFlowNetwork(final Net source, final String sourceLabel, final String targetLabel)
			throws PuckException {
		Graph<Cluster<Individual>> result;

		if ((source == null) || (sourceLabel == null) || (targetLabel == null)) {
			throw new NullPointerException("Null parameter detected.");
		} else {

			//
			Partition<Individual> sourcePartition = PartitionMaker.createRaw(source, sourceLabel);
			Partition<Individual> targetPartition = PartitionMaker.createRaw(source, targetLabel);

			//
			result = new Graph<Cluster<Individual>>("Unmerged Flow Network: " + source.getLabel() + " " + sourceLabel + " " + targetLabel);

			//
			for (Individual individual : source.individuals()) {
				//
				Cluster<Individual> sourceCluster = sourcePartition.getCluster(individual);
				Cluster<Individual> targetCluster = targetPartition.getCluster(individual);

				//
				if ((sourceCluster.getLabel() != null) && (targetCluster.getLabel() != null)) {
					result.incArcWeight(sourceCluster, targetCluster);
				}
			}
		}

		//
		return result;
	}

	/**
	 * 
	 * @param source
	 * @return
	 */
	static public <E> AllianceNet graphToAllianceNet(final Graph<E> source) {
		AllianceNet result;

		//
		Matrix matrix = source.getSquareMatrix().getMatrix();

		//
		result = new AllianceNet(true);

		// Add nodes.
		Vector<Group> groups = new Vector<Group>();
		for (int columnIndex = 0; columnIndex < matrix.getColDim(); columnIndex++) {
			Group group = result.addNode();
			groups.add(group);
		}

		//
		for (int rowIndex = 0; rowIndex < matrix.getRowDim(); rowIndex++) {
			//
			for (int columnIndex = 0; columnIndex < matrix.getColDim(); columnIndex++) {
				result.addEdge(groups.get(rowIndex), groups.get(columnIndex), matrix.get(rowIndex, columnIndex));
			}
		}

		//
		return result;
	}

}
