package org.tip.puckgui.models;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.tip.puck.net.Family;
import org.tip.puck.net.Gender;
import org.tip.puck.net.Individual;
import org.tip.puck.net.workers.NetUtils;
import org.tip.puckgui.InputSettings.CheckLevel;
import org.tip.puckgui.NetGUI;
import org.tip.puckgui.PuckGUI;
import org.tip.puckgui.util.ComboBoxIds;

/**
 * 
 * @author TIP
 */
public class IndividualChildrenModel extends AbstractTableModel {
	private static final long serialVersionUID = 7713941886363202385L;
	static private final Logger logger = LoggerFactory.getLogger(IndividualChildrenModel.class);
	private static final ResourceBundle BUNDLE = ResourceBundle.getBundle("org.tip.puckgui.messages");

	private static ImageIcon smallFemaleIcon = new ImageIcon(IndividualChildrenModel.class.getResource("/org/tip/puckgui/images/female-16x16.png"));
	private static ImageIcon smallMaleIcon = new ImageIcon(IndividualChildrenModel.class.getResource("/org/tip/puckgui/images/male-16x16.png"));
	private static ImageIcon smallUnknowIcon = new ImageIcon(IndividualChildrenModel.class.getResource("/org/tip/puckgui/images/unknown-16x16.png"));

	private NetGUI netGUI;
	private Individual source;
	private List<Individual> children;

	/**
	 * 
	 */
	public IndividualChildrenModel(final NetGUI netGUI, final Individual source) {
		super();
		this.netGUI = netGUI;
		this.source = source;
		if (source == null) {
			this.children = new ArrayList<Individual>();
		} else {
			this.children = this.source.getChildren().toSortedList();
		}
	}

	/**
	 * 
	 * @return
	 */
	public List<Individual> children() {
		List<Individual> result;

		result = this.children;

		//
		return result;
	}

	/**
	 * 
	 */
	public void escapeNewEdition() {
		if (isNewEditionOn()) {
			logger.debug("ESCAPE d");
			int index = this.children.size() - 1;
			this.children.remove(index);
			fireTableRowsDeleted(index, index);
		}
	}

	/**
	 * 
	 */
	@Override
	public Class getColumnClass(final int columnIndex) {
		Class result;

		switch (columnIndex) {
			case 0:
				result = Number.class;
			break;
			case 1:
				result = ImageIcon.class;
			break;
			case 2:
				result = String.class;
			break;
			case 3:
				result = String.class;
			break;
			case 4:
				result = String.class;
			break;
			case 5:
				result = String.class;
			break;
			default:
				result = String.class;
		}

		//
		return result;
	}

	/**
	 * 
	 */
	@Override
	public int getColumnCount() {
		int result;

		result = 6;

		//
		return result;
	}

	/**
	 * 
	 */
	@Override
	public String getColumnName(final int columnIndex) {
		String result;

		switch (columnIndex) {
			case 0:
				result = BUNDLE.getString("MainWindow.individual.id");
			break;
			case 1:
				result = BUNDLE.getString("MainWindow.individual.gender");
				;
			break;
			case 2:
				result = BUNDLE.getString("MainWindow.individual.name");
				;
			break;
			case 3:
				result = BUNDLE.getString("MainWindow.individual.birth");
				;
			break;
			case 4:
				result = BUNDLE.getString("MainWindow.individual.death");
				;
			break;
			case 5:
				result = BUNDLE.getString("MainWindow.individual.otherparent");
				;
			break;
			default:
				result = "";
		}

		//
		return result;
	}

	/**
	 * 
	 */
	@Override
	public int getRowCount() {
		int result;

		result = this.children.size();

		//
		return result;
	}

	/**
	 * 
	 * @return
	 */
	public Individual getSource() {
		Individual result;

		result = this.source;

		//
		return result;
	}

	/**
	 * 
	 */
	@Override
	public Object getValueAt(final int rowIndex, final int columnIndex) {
		Object result;

		if (this.source == null) {
			result = null;
		} else if ((isNewEditionOn() && (rowIndex == this.children.size() - 1))) {
			switch (columnIndex) {
				case 5:
					Individual otherParent = this.children.get(rowIndex).getOtherParent(this.source);
					if (otherParent == null) {
						result = null;
					} else {
						result = String.format("%c (%d) %s", otherParent.getGender().toChar(), otherParent.getId(), otherParent.getName());
					}
				break;
				default:
					result = "";
			}
		} else {
			switch (columnIndex) {
				case 0:
					result = this.children.get(rowIndex).getId();
				break;
				case 1:
					switch (this.children.get(rowIndex).getGender()) {
						case FEMALE:
							result = smallFemaleIcon;
						break;
						case MALE:
							result = smallMaleIcon;
						break;
						case UNKNOWN:
							result = smallUnknowIcon;
						break;
						default:
							result = smallUnknowIcon;
					}
				break;
				case 2:
					result = this.children.get(rowIndex).getName();
				break;
				case 3:
					result = null;
				break;
				case 4:
					result = null;
				break;
				case 5:
					Individual otherParent = this.children.get(rowIndex).getOtherParent(this.source);
					if (otherParent == null) {
						result = null;
					} else {
						result = String.format("%c (%d) %s", otherParent.getGender().toChar(), otherParent.getId(), otherParent.getName());
					}
				break;
				default:
					result = null;
			}
		}

		//
		return result;
	}

	/**
	 * 
	 */
	@Override
	public boolean isCellEditable(final int row, final int col) {
		boolean result;

		switch (col) {
			case 0:
				result = true;
			break;

			default:
				result = false;
		}

		//
		return result;
	}

	/**
	 * This method indicates if a new line is editing.
	 * 
	 * @return
	 */
	public boolean isNewEditionOn() {
		boolean result;

		if (this.source == null) {
			result = false;
		} else if (this.children.size() == this.source.getChildren().size() + 1) {
			result = true;
		} else {
			result = false;
		}

		//
		return result;
	}

	/**
	 * 
	 */
	public void setNewItem(final Family family) {

		Individual newIndividual = new Individual(0, "?", Gender.UNKNOWN);
		newIndividual.setOriginFamily(family);
		this.children.add(newIndividual);
		fireTableDataChanged();
	}

	/**
	 * 
	 * @param source
	 */
	public void setSource(final Individual source) {
		this.source = source;
		if (source == null) {
			this.children = new ArrayList<Individual>();
		} else {
			this.children = this.source.getChildren().toSortedList();
		}
		fireTableDataChanged();
	}

	/**
	 * 
	 */
	@Override
	public void setValueAt(final Object value, final int rowIndex, final int columnIndex) {
		logger.debug("setValueAt(" + rowIndex + ", " + columnIndex + ", " + value + ")");

		//
		if (this.netGUI != null) {
			if (columnIndex == 0) {
				// In case of new child created with id lesser than
				// the current individual, the current index
				// value becomes wrong. So we need to save the
				// previous one.
				Individual currentIndividual = this.netGUI.selectedIndividual();

				if (isNewEditionOn()) {
					Individual newChild = ComboBoxIds.getOrCreateIndividualFromInput(this.netGUI.getNet(), value, Gender.UNKNOWN);
					if (newChild == null) {
						this.children.remove(rowIndex);
						fireTableDataChanged();
					} else {
						Family targetFamily = this.children.get(rowIndex).getOriginFamily();

						// Check errors.
						String errorMessage = controlChildKin(CheckLevel.ERROR, this.source, targetFamily, newChild);
						if (errorMessage != null) {
							JOptionPane.showMessageDialog(null, errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
						} else {
							// Check warning.
							String warningMessage = controlChildKin(CheckLevel.WARNING, this.source, targetFamily, newChild);
							if (warningMessage != null) {
								JOptionPane.showMessageDialog(null, warningMessage, "Warning", JOptionPane.WARNING_MESSAGE);
							}

							// Check if the family is already existing.
							logger.debug("halfO=" + newChild.isHalfOrphan());
							logger.debug("isParent=" + newChild.isParent(this.source));
							if ((!newChild.isHalfOrphan()) || (!newChild.isParent(this.source))) {
								//
								this.netGUI.getNet().removeChild(newChild);
								if (targetFamily == null) {

									//
									if (NetUtils.isRolesFixedByGender(this.source, null)) {
										netGUI.getNet().createFamily(this.source, null, newChild);
									} else {
										netGUI.getNet().createFamily((Individual) null, this.source, newChild);
									}
								} else {
									netGUI.getNet().addChild(targetFamily, newChild);
								}

								//
								this.netGUI.setChanged(true);
								netGUI.updateAll();

								// In case of new child created with id lesser
								// than
								// the current individual, the current index
								// value becomes wrong. So we need to set the
								// previous one.
								this.netGUI.selectIndividualsTab(currentIndividual);
							}
						}
					}
				} else {
					//
					Individual oldChild = this.children.get(rowIndex);
					Individual newChild = ComboBoxIds.getOrCreateIndividualFromInput(this.netGUI.getNet(), value, Gender.UNKNOWN);
					Family targetFamily = this.children.get(rowIndex).getOriginFamily();

					//
					if (newChild == null) {
						this.netGUI.getNet().removeChild(oldChild);
						this.netGUI.setChanged(true);
						setSource(this.source);
					} else {
						// Check errors.
						String errorMessage = controlChildKin(CheckLevel.ERROR, this.source, targetFamily, newChild);
						if (errorMessage != null) {
							JOptionPane.showMessageDialog(null, errorMessage, "Error", JOptionPane.ERROR_MESSAGE);
						} else {
							// Check warning.
							String warningMessage = controlChildKin(CheckLevel.WARNING, this.source, targetFamily, newChild);
							if (warningMessage != null) {
								JOptionPane.showMessageDialog(null, warningMessage, "Warning", JOptionPane.WARNING_MESSAGE);
							}

							if (newChild != oldChild) {
								//
								Family currentFamily = oldChild.getOriginFamily();
								this.netGUI.getNet().removeChild(oldChild);
								this.netGUI.getNet().removeChild(newChild);
								this.netGUI.getNet().addChild(currentFamily, newChild);
								this.netGUI.setChanged(true);
								netGUI.updateAll();

								// In case of new child created with id lesser
								// than
								// the current individual, the current index
								// value becomes wrong. So we need to set the
								// previous one.
								this.netGUI.selectIndividualsTab(currentIndividual);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * 
	 * @return
	 */
	public Individual source() {
		Individual result;

		result = this.source;

		//
		return result;
	}

	/**
	 * 
	 * @param level
	 * @param ego
	 * @param child
	 * @return
	 */
	public static String controlChildKin(final CheckLevel level, final Individual ego, final Family family, final Individual child) {
		String result;

		//
		Individual alter;
		if (family == null) {
			alter = null;
		} else {
			alter = family.getOtherParent(ego);
		}

		//
		if ((level == CheckLevel.ERROR) && ((NetUtils.isSame(ego, child)) || ((NetUtils.isSame(alter, child))))) {
			result = "Same parent child detected.";
		} else if ((level == CheckLevel.ERROR) && (family != null) && (family.getChildren().contains(child))) {
			result = "Already child detected.";
		} else if ((PuckGUI.instance().getPreferences().getInputSettings().getParentChildMarriages() == level)
				&& (((NetUtils.isParentChildMarriage(ego, child)) || (NetUtils.isParentChildMarriage(alter, child))))) {
			result = "Parent-child marriages detected.";
		} else {
			result = null;
		}

		//
		return result;
	}
}
